﻿using DinerContracts.BindingModels;
using DinerContracts.BusinessLogicsContracts;
using DinerDataModels.Enum;
using Microsoft.Extensions.Logging;

namespace Diner
{
	public partial class FormMain : Form
	{
		private readonly ILogger _logger;

		private readonly IOrderLogic _orderLogic;

		private readonly IReportLogic _reportLogic;
		private readonly IWorkProcess _workProcess;
		public FormMain(ILogger<FormMain> logger, IOrderLogic orderLogic, IReportLogic reportLogic, IWorkProcess workProcess)
		{
			InitializeComponent();
			_logger = logger;
			_orderLogic = orderLogic;
			_reportLogic = reportLogic;
			_workProcess = workProcess;
		}
		private void FormMain_Load(object sender, EventArgs e)
		{
			LoadData();
		}
		private void LoadData()
		{
			// прописать логику
			try
			{
				var list = _orderLogic.ReadList(null);
				if (list != null)
				{
					dataGridView.DataSource = list;
					dataGridView.Columns["Id"].Visible = false;
					dataGridView.Columns["SnackId"].Visible = false;
					dataGridView.Columns["ImplementerId"].Visible = false;
					dataGridView.Columns["ClientId"].Visible = false;
					dataGridView.Columns["SnackName"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
					dataGridView.Columns["ClientFIO"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
				}
				_logger.LogInformation("Загрузка заказов");
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки списка заказов");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}
		private void ComponentToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormComponents));
			if (service is FormComponents form)
			{
				form.ShowDialog();
			}
		}
		private void ProductToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormSnacks));
			if (service is FormSnacks form)
			{
				form.ShowDialog();
			}
		}
		private void ButtonCreateOrder_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormCreateOrder));
			if (service is FormCreateOrder form)
			{
				form.ShowDialog();
				LoadData();
			}
		}
	
		
		private void ButtonIssuedOrder_Click(object sender, EventArgs e)
		{
			if (dataGridView.SelectedRows.Count == 1)
			{
				int id = Convert.ToInt32(dataGridView.SelectedRows[0].Cells["Id"].Value);
				_logger.LogInformation("Заказ №{id}. Меняется статус на 'Выдан'", id);
				try
				{
					var operationResult = _orderLogic.DeliveryOrder(new OrderBindingModel
					{
						Id = id
					});
					if (!operationResult)
					{
						throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
					}
					_logger.LogInformation("Заказ №{id} выдан", id);
					LoadData();
				}
				catch (Exception ex)
				{
					_logger.LogError(ex, "Ошибка отметки о выдачи заказа");
					MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			}
		}
		private void ButtonRef_Click(object sender, EventArgs e)
		{
			LoadData();
		}
		private void ComponentsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			using var dialog = new SaveFileDialog { Filter = "docx|*.docx" };
			if (dialog.ShowDialog() == DialogResult.OK)
			{
				_reportLogic.SaveComponentsToWordFile(new ReportBindingModel { FileName = dialog.FileName });
				MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}

		private void ComponentSnacksToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormReportSnackComponents));
			if (service is FormReportSnackComponents form)
			{
				form.ShowDialog();
			}
		}

		private void OrdersToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormReportOrders));
			if (service is FormReportOrders form)
			{
				form.ShowDialog();
			}
		}
		private void ClientsToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormClients));

			if (service is FormClients form)
			{
				form.ShowDialog();
			}
		}

		private void ImplementersToolStripMenuItem_Click(object sender, EventArgs e)
		{
			var service = Program.ServiceProvider?.GetService(typeof(FormImplementers));

			if (service is FormImplementers form)
			{
				form.ShowDialog();
			}
		}

		private void WorkProcessToolStripDropDownButton1_Click(object sender, EventArgs e)
		{
			_workProcess.DoWork((Program.ServiceProvider?.GetService(typeof(IImplementerLogic)) as IImplementerLogic)!, _orderLogic);

			MessageBox.Show("Процесс обработки запущен", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
		}
	}
}
