﻿using DinerContracts.BindingModels;
using DinerContracts.SearchModels;
using DinerContracts.StoragesContracts;
using DinerContracts.ViewModels;
using DinerDataBaseImplement.Models;
using Microsoft.EntityFrameworkCore;


namespace DinerDataBaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public List<OrderViewModel> GetFullList()
        {
            using var context = new DinerDatabase();

            return context.Orders
                    .Include(x => x.Snack)
                     .Include(x => x.Client)
                   .Include(x => x.Implementer)
                    .Select(x => x.GetViewModel)
                    .ToList();
        }
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            using var context = new DinerDatabase();
            if (!model.Id.HasValue && model.DateFrom.HasValue && model.DateTo.HasValue)
            {
                return context.Orders
                    .Include(x => x.Snack)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else if (model.Id.HasValue)
            {
                return context.Orders
                    .Include(x => x.Snack)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .Where(x => x.Id == model.Id)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else if (model.ClientId.HasValue)
            {
                return context.Orders
                       .Include(x => x.Snack)
                       .Include(x => x.Client)
                       .Include(x => x.Implementer)
                       .Where(x => x.ClientId == model.ClientId)
                       .Select(x => x.GetViewModel)
                       .ToList();
            }
            else if (model.ImplementerId.HasValue)
            {
                return context.Orders
                       .Include(x => x.Snack)
                       .Include(x => x.Client)
                       .Include(x => x.Implementer)
                       .Where(x => x.ImplementerId == model.ImplementerId)
                       .Select(x => x.GetViewModel)
                       .ToList();
            }

            return context.Orders
                  .Include(x => x.Snack)
                  .Include(x => x.Client)
                  .Include(x => x.Implementer)
                  .Where(x => model.Status == x.Status)
                  .Select(x => x.GetViewModel)
                  .ToList();
        }
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new DinerDatabase();
            return context.Orders.Include(x => x.Snack)
                 .Include(x => x.Client)
             .FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)?.GetViewModel;
        }
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            using var context = new DinerDatabase();
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return context.Orders.Include(x => x.Snack)
                                 .Include(x => x.Client)
                .FirstOrDefault(x => x.Id == newOrder.Id)?.GetViewModel; ;
        }
        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new DinerDatabase();
                var order = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
                if (order == null)
                {
                    return null;
                }
                order.Update(model);
                context.SaveChanges();

                return context.Orders
                    .Include(x => x.Snack)
                     .Include(x => x.Client)
                    .FirstOrDefault(x => x.Id == model.Id)?
                    .GetViewModel;
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new DinerDatabase();
            var element = context.Orders
            .FirstOrDefault(x => x.Id == model.Id);

            if (element != null)
            {
                var deletedElement = context.Orders
                    .Include(x => x.Snack)
                    .Include(x => x.Client)
                    .FirstOrDefault(x => x.Id == model.Id)
                    ?.GetViewModel;
                context.Orders.Remove(element);
                context.SaveChanges();
                return deletedElement;
            }
            return null;
        }
    }
}
