﻿using DinerContracts.BindingModels;
using DinerContracts.ViewModels;
using DinerDataModels.Enum;
using DinerDataModels.Models;
using System.Reflection;
using System.Xml.Linq;

namespace DinerFileImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        public int SnackID { get; private set; }
        public int ClientId { get; private set; }
        public int? ImplementerId { get; private set; }

        public int Count { get; private set; }

        public double Sum { get; private set; }

        public OrderStatus Status { get; private set; }

        public DateTime DateCreate { get; private set; }

        public DateTime? DateImplement { get; private set; }
        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                SnackID = model.SnackId,
                ClientId = model.ClientId,
                ImplementerId = model.ImplementerId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }
        public static Order? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            var order = new Order()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                SnackID = Convert.ToInt32(element.Element("SnackID")!.Value),
                ClientId = Convert.ToInt32(element.Element("ClientId")!.Value),
                ImplementerId = Convert.ToInt32(element.Element("ImplementerId")!.Value),
                Count = Convert.ToInt32(element.Element("Count")!.Value),
                Sum = Convert.ToDouble(element.Element("Sum")!.Value),
                Status = (OrderStatus)Enum.Parse(typeof(OrderStatus), element.Element("Status")!.Value),
                DateCreate = Convert.ToDateTime(element.Element("DateCreate")!.Value),
            };
            DateTime.TryParse(element.Element("DateImplement")!.Value, out DateTime _dateImplement);
            order.DateImplement = _dateImplement;
            return order;
        }
        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }

            Status = model.Status;
            DateImplement = model.DateImplement;
        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            SnackId = SnackID,
            ClientId = ClientId,
            ImplementerId = ImplementerId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };
        public XElement GetXElement => new("Order",
            new XAttribute("Id", Id),
            new XElement("SnackID", SnackID.ToString()),
              new XElement("ClientId", ClientId.ToString()),
           new XElement("ImplementerId", ImplementerId.ToString()),
            new XElement("Count", Count.ToString()),
            new XElement("Sum", Sum.ToString()),
            new XElement("Status", Status.ToString()),
            new XElement("DateCreate", DateCreate.ToString()),
            new XElement("DateImplement", DateImplement?.ToString())
            );
    }
}
